/* test1
*
*/
#define WIN32_LEAN_AND_MEAN
#include <stdlib.h>
#include <windows.h>
#include "applic.h"

class Module
{
public:
	static HINSTANCE hInstance;
	static PSTR szCmdLine;
	static int iCmdShow;
	static PSTR Name;
	static void RegisterWindow();
};

///////////////////////////////////////////////////////////////////////////////
// CAppWindow class

CAppWindow::CAppWindow(PSTR szName)
{
	Module::Name = Name = szName; // classname = Window caption, for simplicity
	hpal = 0;
	Width = Height = CW_USEDEFAULT;
}

CAppWindow::~CAppWindow()
{
	if (hpal) DeleteObject(hpal);
}

void CAppWindow::Create()
{
	Module::RegisterWindow();
	/*
	* Pass 'this' pointer in lpParam of CreateWindow()
	* The first time WM_CREATE is called, WndProc will
	* store it in the "extra" bytes reserved for each instance
	* of the program (see RegisterWindow). WndProc uses this information
	* to call the MsgProc member function of the object.
	*/
	hWnd = CreateWindow(
		Module::Name, // window class name
		Name,         // window caption
		WS_OVERLAPPEDWINDOW, // window style
						     // | WS_VSCROLL if vertical scroll needed
		CW_USEDEFAULT,  // initial x position;
		CW_USEDEFAULT,  // initial y position;
		Width + 2*GetSystemMetrics(SM_CXFRAME),   // inital x size
		Height+ 2*GetSystemMetrics(SM_CYFRAME)
			    + GetSystemMetrics(SM_CYCAPTION), // initial y size
		NULL,   // parent window handle
		NULL,   // window menu handle
		Module::hInstance, // program instance handle
		(PSTR) this // creation parameters
	);
	if (!hWnd) exit(-1);
	SetupWindow();
 	ShowWindow(hWnd, Module::iCmdShow);
	UpdateWindow(hWnd);
}

int CAppWindow::Run()
{
	MSG eventMessage;
	Create();
	// Keep running until the user kills the program
	while (GetMessage(&eventMessage, 0, 0, 0)) {
		TranslateMessage(&eventMessage);
		DispatchMessage(&eventMessage);
	}
	return eventMessage.wParam;
}

void CAppWindow::Destroy()
{
	ShowWindow(hWnd, SW_HIDE);
	CleanupWindow();
	PostQuitMessage(0);
}


void CAppWindow::Invalidate(BOOL Flag)
{
	InvalidateRect(hWnd, 0, Flag);
}

long CAppWindow::MsgProc(UINT iMessage, UINT wParam, LONG lParam)
{
	PAINTSTRUCT ps;
	HDC hdc;
	POINT pt;
	switch (iMessage) {
		case WM_CREATE:
			break;
		case WM_SIZE:
			WmSize(LOWORD(lParam),HIWORD(lParam));
			break;
		case WM_SETFOCUS:
			WmSetFocus();
			break;
		case WM_KILLFOCUS:
			WmKillFocus();
			break;
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			Paint(hdc);
			EndPaint(hWnd, &ps);
			break;
		case WM_CHAR:
			WmChar(wParam, lParam);
			break;
		case WM_COMMAND:
			WmCommand(wParam, lParam);
			break;
		case WM_DRAWITEM:
			WmDrawItem(wParam, lParam);
			break;
		case WM_KEYDOWN:
			WmKeyDown(wParam, lParam);
			break;
		case WM_LBUTTONDOWN:
			pt.x = LOWORD(lParam);
			pt.y = HIWORD(lParam);
			WmLButtonDown(pt);
			break;
		case WM_LBUTTONUP:
			pt.x = LOWORD(lParam);
			pt.y = HIWORD(lParam);
			WmLButtonUp(pt);
			break;
		case WM_MOUSEMOVE:
			pt.x = LOWORD(lParam);
			pt.y = HIWORD(lParam);
			WmMouseMove(wParam, pt);
			break;
		case WM_TIMER:
			WmTimer();
			break;
		case WM_VSCROLL:
			WmVScroll(wParam,lParam);
			break;
		case WM_QUERYNEWPALETTE:
			SetPalette(FALSE); // Realize in the foreground
			break;
		case WM_PALETTECHANGED:
			// See if the change was caused by us and ignore it if not.
			if ( (HWND) wParam != hWnd) {
				SetPalette(TRUE); // Realize in the background. 
			}
			break;
		case WM_DESTROY:
			Destroy();
			break;
		default:
			return DefWindowProc(hWnd, iMessage, wParam, lParam);
	}
	return 0;
}

// helper routine to keep palette updated

int CAppWindow::SetPalette(BOOL bForceBkgnd)
{
	if (!hpal) return 0;
	// We are going active, so realize our palette.
	HDC hdc = GetDC(hWnd);
	HPALETTE oldpalette = SelectPalette(hdc,hpal,bForceBkgnd);
	int NumColorsChanged = RealizePalette(hdc);
	SelectPalette(hdc, oldpalette, TRUE);
	RealizePalette(hdc);
	ReleaseDC(hWnd,hdc);
	// If any colors have changed or we are in the
	// background, repaint the lot.
	if (NumColorsChanged || bForceBkgnd) {
		InvalidateRect(hWnd, NULL, TRUE); // Repaint.
	}
	return NumColorsChanged;
}

// helper routines to access extra bytes in window

inline CAppWindow *GetPointer(HWND hWnd)
{
	return (CAppWindow *) GetWindowLong(hWnd, 0);
}

inline void SetPointer(HWND hWnd, CAppWindow *pWindow)
{
	SetWindowLong(hWnd, 0, (LONG) pWindow);
}

// Routines to access static Module data

HINSTANCE GetInstance()
{
	return Module::hInstance;
}

PSTR GetCmdLine()
{
	return Module::szCmdLine;
}

/////////////////////////////////////////////////////////////////////////
// Module class implementation


HINSTANCE Module::hInstance;
PSTR Module::szCmdLine;
int Module::iCmdShow;
PSTR Module::Name;

LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam);

void Module::RegisterWindow()
{
	WNDCLASS wndclass;
	PSTR szRes = "APPLIC";
	HICON hicon = LoadIcon(hInstance,szRes);
	HCURSOR hcursor = LoadCursor(hInstance,szRes);

	//wndclass.cbSize = sizeof(wndclass);
	wndclass.style = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc = WndProc;
	wndclass.cbClsExtra = 0;
	/* Reserve extra bytes for a pointer to the C++
	*  (CappWindow) object corresponding to the window.
	*/
	wndclass.cbWndExtra = sizeof (CAppWindow *);
	wndclass.hInstance = hInstance;
	wndclass.hIcon = (hicon? hicon: LoadIcon(0,IDI_APPLICATION));
	wndclass.hCursor =  (hcursor? hcursor: LoadCursor(0,IDC_ARROW));
	wndclass.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName = szRes;
	wndclass.lpszClassName = Name;

	int hAtom = RegisterClass(&wndclass);
	if (!hAtom) exit(-1);
}

//////////////////////////////////////////////////////////////////////////////////
// Message Handler for this instance

LRESULT CALLBACK WndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	CAppWindow *pWindow = GetPointer(hWnd);
	if (pWindow) {
		return pWindow->MsgProc(iMessage, wParam, lParam);
	}
	else {
		/*
		*  Store a pointer to this object in the window's extra
		*  bytes.  This will enable us to acess this object (and
		*  its member functions) where we are only given
		*  a handle to identify the window.
		*/
		if (iMessage== WM_CREATE) {
			LPCREATESTRUCT lpcs;
			lpcs = (LPCREATESTRUCT) lParam;
			pWindow = (CAppWindow *) lpcs->lpCreateParams;
			SetPointer(hWnd, pWindow);
			return pWindow->MsgProc(iMessage, wParam, lParam);
		}
		else
			return DefWindowProc(hWnd, iMessage, wParam, lParam);
	}
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
				   PSTR szCmdLine, int iCmdShow)
{
	extern int Main();

	Module::hInstance = hInstance;
	Module::szCmdLine = szCmdLine;
	Module::iCmdShow = iCmdShow;
	return Main();
}
